@@ -82,6 +82,7 @@ static void realloc_members();
 // Bounding box to filter imported data
 const char *bbox = NULL;
 static double minlon, minlat, maxlon, maxlat;
+static int extra_attributes;
 
 static void printStatus(void)
 {
@@ -301,6 +302,38 @@ void StartElement(xmlTextReaderPtr reader, const xmlChar *name)
     } else {
         fprintf(stderr, "%s: Unknown element name: %s\n", __FUNCTION__, name);
     }
+
+    // Collect extra attribute information and add as tags
+    if (extra_attributes && (xmlStrEqual(name, BAD_CAST "node") ||
+                             xmlStrEqual(name, BAD_CAST "way") ||
+                             xmlStrEqual(name, BAD_CAST "relation")))
+    {
+        xmlChar *xtmp;
+
+        xtmp = xmlTextReaderGetAttribute(reader, BAD_CAST "user");
+        if (xtmp) {
+            addItem(&tags, "osm_user", (char *)xtmp, 0);
+            xmlFree(xtmp);
+        }
+
+        xtmp = xmlTextReaderGetAttribute(reader, BAD_CAST "uid");
+        if (xtmp) {
+            addItem(&tags, "osm_uid", (char *)xtmp, 0);
+            xmlFree(xtmp);
+        }
+
+        xtmp = xmlTextReaderGetAttribute(reader, BAD_CAST "version");
+        if (xtmp) {
+            addItem(&tags, "osm_version", (char *)xtmp, 0);
+            xmlFree(xtmp);
+        }
+
+        xtmp = xmlTextReaderGetAttribute(reader, BAD_CAST "timestamp");
+        if (xtmp) {
+            addItem(&tags, "osm_timestamp", (char *)xtmp, 0);
+            xmlFree(xtmp);
+        }
+    }
 }
 
 static void resetMembers()
@@ -376,6 +409,7 @@ void EndElement(const xmlChar *name)
         /* ignore */
     } else if (xmlStrEqual(name, BAD_CAST "changeset")) {
         /* ignore */
+        resetList(&tags); /* We may have accumulated some tags even if we ignored the changeset */
     } else if (xmlStrEqual(name, BAD_CAST "add")) {
         action = ACTION_NONE;
     } else if (xmlStrEqual(name, BAD_CAST "create")) {
@@ -489,7 +523,7 @@ static void long_usage(char *arg0)
     fprintf(stderr, "   -p|--prefix\t\tPrefix for table names (default planet_osm)\n");
     fprintf(stderr, "   -s|--slim\t\tStore temporary data in the database. This greatly\n");
     fprintf(stderr, "            \t\treduces the RAM usage but is much slower.\n");
-    fprintf(stderr, "   -S|--style\t\tLocation of the style file. Defaults to ./default.style\n");
+    fprintf(stderr, "   -S|--style\t\tLocation of the style file. Defaults to /usr/share/osm2pgsql/default.style\n");
     fprintf(stderr, "   -C|--cache\t\tOnly for slim mode: Use upto this many MB for caching nodes\n");
     fprintf(stderr, "             \t\tDefault is 800\n");
     fprintf(stderr, "   -U|--username\tPostgresql user name.\n");
@@ -500,7 +534,13 @@ static void long_usage(char *arg0)
     fprintf(stderr, "   -o|--expire-output filename\tOutput filename for expired tiles list.\n");
     fprintf(stderr, "   -O|--output\t\tOutput backend.\n");
     fprintf(stderr, "              \t\tpgsql - Output to a PostGIS database. (default)\n");
+    fprintf(stderr, "              \t\tgazetteer - Output to a PostGIS database suitable for gazetteer\n");
     fprintf(stderr, "              \t\tnull  - No output. Useful for testing.\n");
+    fprintf(stderr, "   -x|--extra-attributes\n");
+    fprintf(stderr, "              \t\tInclude attributes for each object in the database.\n");
+    fprintf(stderr, "              \t\tThis includes the username, userid, timestamp and version.\n"); 
+    fprintf(stderr, "              \t\tNote: this option also requires additional entries in your style file.\n"); 
+    fprintf(stderr, "   -G|--multi-geometry\t\tGenerate multi-geometry features in postgresql tables.\n");
     fprintf(stderr, "   -h|--help\t\tHelp information.\n");
     fprintf(stderr, "   -v|--verbose\t\tVerbose output.\n");
     fprintf(stderr, "\n");
@@ -589,10 +629,12 @@ int main(int argc, char *argv[])
     int create=0;
     int slim=0;
     int sanitize=0;
+    int long_usage_bool=0;
     int pass_prompt=0;
     int projection = PROJ_SPHERE_MERC;
     int expire_tiles_zoom = -1;
     int expire_tiles_zoom_min = -1;
+    int enable_multi = 0;
     const char *expire_tiles_filename = "dirty_tiles";
     const char *db = "gis";
     const char *username=NULL;
@@ -601,7 +643,7 @@ int main(int argc, char *argv[])
     const char *port = "5432";
     const char *conninfo = NULL;
     const char *prefix = "planet_osm";
-    const char *style = "./default.style";
+    const char *style = "/usr/share/osm2pgsql/default.style";
     const char *temparg;
     const char *output_backend = "pgsql";
     int cache = 800;
@@ -634,11 +676,13 @@ int main(int argc, char *argv[])
             {"style",    1, 0, 'S'},
             {"expire-tiles", 1, 0, 'e'},
             {"expire-output", 1, 0, 'o'},
-	    {"output",   1, 0, 'O'},
+            {"output",   1, 0, 'O'},
+            {"extra-attributes", 0, 0, 'x'},
+            {"multi-geometry", 0, 0, 'G'},
             {0, 0, 0, 0}
         };
 
-        c = getopt_long (argc, argv, "ab:cd:hlmMp:suvU:WH:P:E:C:S:e:o:O:", long_options, &option_index);
+        c = getopt_long (argc, argv, "ab:cd:hlmMp:suvU:WH:P:E:C:S:e:o:O:xG", long_options, &option_index);
         if (c == -1)
             break;
 
@@ -669,11 +713,9 @@ int main(int argc, char *argv[])
                 break;
             case 'o': expire_tiles_filename=optarg; break;
 	    case 'O': output_backend = optarg; break;
-
-            case 'h':
-                long_usage(argv[0]);
-                exit(EXIT_FAILURE);
-
+            case 'x': extra_attributes=1; break;
+            case 'G': enable_multi=1; break;
+            case 'h': long_usage_bool=1; break;
             case '?':
             default:
                 short_usage(argv[0]);
@@ -681,6 +723,11 @@ int main(int argc, char *argv[])
         }
     }
 
+    if (long_usage_bool) {
+        long_usage(argv[0]);
+        exit(EXIT_FAILURE);
+    }
+
     if (argc == optind) {  // No non-switch arguments
         short_usage(argv[0]);
         exit(EXIT_FAILURE);
@@ -691,11 +738,6 @@ int main(int argc, char *argv[])
         exit(EXIT_FAILURE);
     }
 
-    if ((expire_tiles_zoom >= 0) && (projection != PROJ_SPHERE_MERC)) {
-        fprintf(stderr, "Error: --expire-tiles can only be used with spherical mercator projection!\n");
-        exit(EXIT_FAILURE);
-    }
-    
     if( cache < 0 ) cache = 0;
 
     if (pass_prompt)
@@ -737,6 +779,7 @@ int main(int argc, char *argv[])
     options.expire_tiles_zoom = expire_tiles_zoom;
     options.expire_tiles_zoom_min = expire_tiles_zoom_min;
     options.expire_tiles_filename = expire_tiles_filename;
+    options.enable_multi = enable_multi;
 
     if (strcmp("pgsql", output_backend) == 0) {
       out = &out_pgsql;
@@ -745,7 +788,7 @@ int main(int argc, char *argv[])
     } else if (strcmp("null", output_backend) == 0) {
       out = &out_null;
     } else {
-      fprintf(stderr, "Output backend `%s' not recognised. Should be one of [pgsql, null].\n", output_backend);
+      fprintf(stderr, "Output backend `%s' not recognised. Should be one of [pgsql, gazetteer, null].\n", output_backend);
       exit(EXIT_FAILURE);
     }
 
