@@ -30,7 +30,10 @@
 
 #define SRID (project_getprojinfo()->srs)
 
-#define MAX_STYLES 100
+/* FIXME: Shouldn't malloc this all to begin with but call realloc()
+   as required. The program will most likely segfault if it reads a
+   style file with more styles than this */
+#define MAX_STYLES 1000
 
 enum table_id {
     t_point, t_line, t_poly, t_roads
@@ -51,7 +54,7 @@ static struct s_table {
 } tables [] = {
     { name: "%s_point",   type: "POINT"     },
     { name: "%s_line",    type: "LINESTRING"},
-    { name: "%s_polygon", type: "POLYGON"  },
+    { name: "%s_polygon", type: "GEOMETRY"  }, // Actually POLGYON & MULTIPOLYGON but no way to limit to just these two
     { name: "%s_roads",   type: "LINESTRING"}
 };
 #define NUM_TABLES ((signed)(sizeof(tables) / sizeof(tables[0])))
@@ -118,6 +121,7 @@ void read_style_file( const char *filename )
 {
   FILE *in;
   int lineno = 0;
+  int num_read = 0;
 
   exportList[OSMTYPE_NODE] = malloc( sizeof(struct taginfo) * MAX_STYLES );
   exportList[OSMTYPE_WAY]  = malloc( sizeof(struct taginfo) * MAX_STYLES );
@@ -192,6 +196,15 @@ void read_style_file( const char *filename )
       fprintf( stderr, "Weird style line %d\n", lineno );
       exit_nicely();
     }
+    num_read++;
+  }
+  if (ferror(in)) {
+      perror(filename);
+      exit_nicely();
+  }
+  if (num_read == 0) {
+      fprintf(stderr, "Unable to parse any valid columns from the style file. Aborting.\n");
+      exit_nicely();
   }
   fclose(in);
 }
@@ -325,7 +338,7 @@ static int add_z_order(struct keyval *tags, int *roads)
     return 0;
 }
 
-
+#if 0
 static void fix_motorway_shields(struct keyval *tags)
 {
     const char *highway = getItem(tags, "highway");
@@ -339,7 +352,7 @@ static void fix_motorway_shields(struct keyval *tags)
     if (name && !ref)
         addItem(tags, "ref", name, 0);
 }
-
+#endif
 
 /* Append all alternate name:xx on to the name tag with space sepatators.
  * name= always comes first, the alternates are in no particular order
@@ -617,40 +630,51 @@ E4C1421D5BF24D06053E7DF4940
 static int pgsql_out_way(int id, struct keyval *tags, struct osmNode *nodes, int count, int exists)
 {
     int polygon = 0, roads = 0;
-    char *wkt;
-    double area, interior_lat, interior_lon;
-    
+    int i, wkt_size;
+    double split_at;
+
     /* If the flag says this object may exist already, delete it first */
-    if(exists)
+    if(exists) {
         pgsql_delete_way_from_output(id);
+        Options->mid->way_changed(id);
+    }
 
     if (pgsql_filter_tags(OSMTYPE_WAY, tags, &polygon) || add_z_order(tags, &roads))
         return 0;
 
-    //compress_tag_name(tags);
+    // Split long ways after around 1 degree or 100km
+    if (Options->projection == PROJ_LATLONG)
+        split_at = 1;
+    else
+        split_at = 100 * 1000;
 
-    fix_motorway_shields(tags);
+    wkt_size = get_wkt_split(nodes, count, polygon, split_at);
 
-    wkt = get_wkt_simple(nodes, count, polygon, &area, &interior_lon, &interior_lat);
-    if (wkt && strlen(wkt)) {
-	/* FIXME: there should be a better way to detect polygons */
-	if (!strncmp(wkt, "POLYGON", strlen("POLYGON"))) {
-            expire_tiles_from_nodes_poly(nodes, count, id);
-	    if (area > 0.0) {
-		char tmp[32];
-		snprintf(tmp, sizeof(tmp), "%f", area);
-		addItem(tags, "way_area", tmp, 0);
-	    }
-	    write_wkts(id, tags, wkt, t_poly);
-	    add_parking_node(id, tags, interior_lat, interior_lon);
-	} else {
-            expire_tiles_from_nodes_line(nodes, count);
-	    write_wkts(id, tags, wkt, t_line);
-	    if (roads)
-		write_wkts(id, tags, wkt, t_roads);
-	}
+    for (i=0;i<wkt_size;i++)
+    {
+        char *wkt = get_wkt(i);
+
+        if (wkt && strlen(wkt)) {
+            /* FIXME: there should be a better way to detect polygons */
+            if (!strncmp(wkt, "POLYGON", strlen("POLYGON")) || !strncmp(wkt, "MULTIPOLYGON", strlen("MULTIPOLYGON"))) {
+                expire_tiles_from_nodes_poly(nodes, count, id);
+                double area = get_area(i);
+                if (area > 0.0) {
+                    char tmp[32];
+                    snprintf(tmp, sizeof(tmp), "%f", area);
+                    addItem(tags, "way_area", tmp, 0);
+                }
+                write_wkts(id, tags, wkt, t_poly);
+            } else {
+                expire_tiles_from_nodes_line(nodes, count);
+                write_wkts(id, tags, wkt, t_line);
+                if (roads)
+                    write_wkts(id, tags, wkt, t_roads);
+            }
+        }
+        free(wkt);
     }
-    free(wkt);
+    clear_wkts();
 	
     return 0;
 }
@@ -658,12 +682,13 @@ static int pgsql_out_way(int id, struct keyval *tags, struct osmNode *nodes, int
 static int pgsql_out_relation(int id, struct keyval *rel_tags, struct osmNode **xnodes, struct keyval *xtags, int *xcount, int *xid, const char **xrole)
 {
     int i, wkt_size;
-    double interior_lat, interior_lon;
     int polygon = 0, roads = 0;
     int make_polygon = 0;
     int make_boundary = 0;
     struct keyval tags, *p, poly_tags;
     char *type;
+    double split_at;
+
 #if 0
     fprintf(stderr, "Got relation with counts:");
     for (i=0; xcount[i]; i++)
@@ -838,7 +863,13 @@ static int pgsql_out_relation(int id, struct keyval *rel_tags, struct osmNode **
         return 0;
     }
 
-    wkt_size = build_geometry(id, xnodes, xcount, make_polygon);
+    // Split long linear ways after around 1 degree or 100km (polygons not effected)
+    if (Options->projection == PROJ_LATLONG)
+        split_at = 1;
+    else
+        split_at = 100 * 1000;
+
+    wkt_size = build_geometry(id, xnodes, xcount, make_polygon, Options->enable_multi, split_at);
 
     if (!wkt_size) {
         resetList(&tags);
@@ -850,10 +881,10 @@ static int pgsql_out_relation(int id, struct keyval *rel_tags, struct osmNode **
     {
         char *wkt = get_wkt(i);
 
-        if (strlen(wkt)) {
+        if (wkt && strlen(wkt)) {
             expire_tiles_from_wkt(wkt, -id);
             /* FIXME: there should be a better way to detect polygons */
-            if (!strncmp(wkt, "POLYGON", strlen("POLYGON"))) {
+            if (!strncmp(wkt, "POLYGON", strlen("POLYGON")) || !strncmp(wkt, "MULTIPOLYGON", strlen("MULTIPOLYGON"))) {
                 double area = get_area(i);
                 if (area > 0.0) {
                     char tmp[32];
@@ -861,14 +892,12 @@ static int pgsql_out_relation(int id, struct keyval *rel_tags, struct osmNode **
                     addItem(&tags, "way_area", tmp, 0);
                 }
                 write_wkts(-id, &tags, wkt, t_poly);
-                get_interior(i, &interior_lat, &interior_lon);
-                add_parking_node(-id, &tags, interior_lat, interior_lon);
-        } else {
+            } else {
                 write_wkts(-id, &tags, wkt, t_line);
                 if (roads)
                     write_wkts(-id, &tags, wkt, t_roads);
             }
-    }
+        }
         free(wkt);
     }
 
@@ -902,7 +931,7 @@ static int pgsql_out_relation(int id, struct keyval *rel_tags, struct osmNode **
     // If we are making a boundary then also try adding any relations which form complete rings
     // The linear variants will have already been processed above
     if (make_boundary) {
-        wkt_size = build_geometry(id, xnodes, xcount, 1);
+        wkt_size = build_geometry(id, xnodes, xcount, 1, Options->enable_multi, split_at);
         for (i=0;i<wkt_size;i++)
         {
             char *wkt = get_wkt(i);
@@ -910,7 +939,7 @@ static int pgsql_out_relation(int id, struct keyval *rel_tags, struct osmNode **
             if (strlen(wkt)) {
                 expire_tiles_from_wkt(wkt, -id);
                 /* FIXME: there should be a better way to detect polygons */
-                if (!strncmp(wkt, "POLYGON", strlen("POLYGON"))) {
+                if (!strncmp(wkt, "POLYGON", strlen("POLYGON")) || !strncmp(wkt, "MULTIPOLYGON", strlen("MULTIPOLYGON"))) {
                     double area = get_area(i);
                     if (area > 0.0) {
                         char tmp[32];
@@ -965,12 +994,7 @@ static int pgsql_out_start(const struct output_options *options)
         tables[i].sql_conn = sql_conn;
 
         if (!options->append) {
-            sprintf( sql, "DROP TABLE %s;", tables[i].name);
-            /* Will be an error if table does not exist and will abort a transaction
-             * DROP TABLE IF EXISTS is not present until PostgreSQL-8.2
-             */
-            res = PQexec(sql_conn, sql);
-            PQclear(res);
+            pgsql_exec(sql_conn, PGRES_COMMAND_OK, "DROP TABLE IF EXISTS %s", tables[i].name);
         }
         else
         {
@@ -991,9 +1015,7 @@ static int pgsql_out_start(const struct output_options *options)
         }
 
         /* These _tmp tables can be left behind if we run out of disk space */
-        sprintf( sql, "DROP TABLE %s_tmp;", tables[i].name);
-        res = PQexec(sql_conn, sql);
-        PQclear(res);
+        pgsql_exec(sql_conn, PGRES_COMMAND_OK, "DROP TABLE IF EXISTS %s_tmp", tables[i].name);
 
         pgsql_exec(sql_conn, PGRES_COMMAND_OK, "BEGIN");
 
@@ -1033,7 +1055,8 @@ static int pgsql_out_start(const struct output_options *options)
             for (j=0; j < numTags; j++) {
                 if( exportTags[j].flags & FLAG_DELETE )
                     continue;
-                if (PQfnumber(res, exportTags[j].name) < 0) {
+                sprintf(tmp, "\"%s\"", exportTags[j].name);
+                if (PQfnumber(res, tmp) < 0) {
 #if 0
                     fprintf(stderr, "Append failed. Column \"%s\" is missing from \"%s\"\n", exportTags[j].name, tables[i].name);
                     exit_nicely();
@@ -1044,7 +1067,12 @@ static int pgsql_out_start(const struct output_options *options)
                 }
                 /* Note: we do not verify the type or delete unused columns */
             }
+
             PQclear(res);
+
+            /* change the type of the geometry column if needed - this can only change to a more permisive type */
+            pgsql_exec(sql_conn, PGRES_COMMAND_OK, "UPDATE geometry_columns SET type = '%s' where type != '%s' and f_table_name = '%s' and f_geometry_column = 'way'",
+                        tables[i].type, tables[i].type, tables[i].name);
         }
         pgsql_exec(sql_conn, PGRES_COMMAND_OK, "PREPARE get_way (int4) AS SELECT AsText(way) FROM %s WHERE osm_id = $1;\n", tables[i].name);
         
